@@ -57,7 +57,7 @@ class MyBroker(bt.BrokerBase):
         position = bt.Position()
 
         symbol = data.p.symbol
-        self._update_positions(force_update=(not self.positions.__contains__(symbol)))
+        self._update_positions(force_update=(not self.positions.__contains__(symbol)) or sync)
         position = self.positions[symbol]
 
         if self.debug:
@@ -218,9 +218,6 @@ class MyBroker(bt.BrokerBase):
         if force_update or datetime.now().timestamp() - self.last_position_update_ts > self.MIN_UPDATE_INTERVAL_SECONDS:
             self.last_position_update_ts = datetime.now().timestamp()
 
-            if self.positions.__len__() > 0:
-                self.positions.clear()
-
             url = self.base_url + f"/positions/{self.name}/{self.id}"
             response = requests.get(url).json()
 
@@ -228,6 +225,8 @@ class MyBroker(bt.BrokerBase):
                 stdout_log(f"{self.__class__.__name__}, _update_positions, positions response: {response}")
 
             if response.get('code') == "200":
+                if self.positions.__len__() > 0:
+                    self.positions.clear()
                 for pos in response['results']:
                     if pos['qty'] != 0:
                         self.positions[pos['symbol']] = bt.Position(size=pos['qty'] if pos['tradeSide'] == 'buy' else -pos['qty'], price=pos['avgPrice'])
