@@ -7,6 +7,7 @@ import time
 import pytz
 import queue
 from ffquant.utils.Logger import stdout_log
+import pandas_market_calendars as pmc
 
 __ALL__ = ['MyLiveFeed']
 
@@ -17,7 +18,7 @@ __ALL__ = ['MyLiveFeed']
 # 如果已经返回了 那就忽略 如果还没有被返回 那就调用API接口获取 并保存到live_data_list中 如果API接口没有获取到数据 那就沿用上一个K线
 class MyLiveFeed(bt.feeds.DataBase):
     params = (
-        ('url', 'http://192.168.25.127:8288/symbol/info/list'),
+        ('url', os.getenv('SYMBOL_INFO_LIST_URL', default='http://192.168.25.127:8285/symbol/info/list')),
         ('symbol', None),
         ('timeframe', bt.TimeFrame.Minutes),
         ('compression', 1),
@@ -25,19 +26,38 @@ class MyLiveFeed(bt.feeds.DataBase):
         ('max_retries', 15),
         ('backpeek_size', 5),
         ('backfill_size', 0),
-        ('check_backfill_size', True)
+        ('check_backfill_size', True),
+        ('market', 'HKEX'),
     )
 
     lines = (('turnover'),)
 
     def __init__(self):
         super(MyLiveFeed, self).__init__()
+        self.pmc = pmc.get_calendar(self.p.market)
+        start_date = datetime.now().strftime('%Y-%m-%d')
+        end_date = (datetime.now() + timedelta(days=1)).strftime('%Y-%m-%d')
+        self.schedule = self.pmc.schedule(start_date=start_date, end_date=end_date)
+
+        self.brk_start_time = datetime.now().replace(hour=12, minute=0, second=0, microsecond=0).astimezone(pytz.utc)
+        self.brk_end_time = datetime.now().replace(hour=13, minute=0, second=0, microsecond=0).astimezone(pytz.utc)
+        self.close_time = datetime.now().replace(hour=16, minute=0, second=0, microsecond=0).astimezone(pytz.utc)
+
+        self.night_open_time = datetime.now().replace(hour=17, minute=30, second=0, microsecond=0).astimezone(pytz.utc)
+        self.night_close_time = (datetime.now().replace(hour=3, minute=0, second=0, microsecond=0) + timedelta(days=1)).astimezone(pytz.utc)
+
         self.live_data_list = list()
         self.hist_data_q = queue.Queue()
 
+        self.dropped_kline_time_list = list()
+
     def islive(self):
         return True
 
+    def is_tradable(self, dt: datetime) -> bool:
+        dt = dt.astimezone(pytz.utc)
+        return dt <= self.brk_start_time or self.brk_end_time < dt <= self.close_time or self.night_open_time < dt <= self.night_close_time
+
     # start方法的调用先于next 但是start方法被调用时 strategy的minperiod还没有被确定 next方法被调用时 strategy的minperiod已经确定
     # 需要根据策略的minperiod来限制backfill_size 所以限制backfill_size需要在next方法中
     def next(self, datamaster=None, ticks=True):
@@ -51,8 +71,8 @@ class MyLiveFeed(bt.feeds.DataBase):
             running_strats = self._env.runningstrats
             max_minperiod = max([strat._minperiod for strat in running_strats])
             if self.p.check_backfill_size and self.p.backfill_size > max_minperiod:
-                stdout_log(f"[CRITICAL], {self.__class__.__name__}, backfill_size({self.p.backfill_size}) > max_minperiod({max_minperiod}), use max_minperiod as backfill_size")
-                self.p.backfill_size = max_minperiod
+                stdout_log(f"[CRITICAL], {self.__class__.__name__}, backfill_size({self.p.backfill_size}) > max_minperiod({max_minperiod}), use max_minperiod - 1 as backfill_size")
+                self.p.backfill_size = max_minperiod - 1
             self.prepare_backfill_data()
         return super().next(datamaster, ticks)
 
@@ -95,6 +115,9 @@ class MyLiveFeed(bt.feeds.DataBase):
             last_time_close = None
             for result in results:
                 time_close = result["timeClose"]
+                if not self.is_tradable(datetime.fromtimestamp(time_close / 1000.0, timezone.utc)):
+                    continue
+
                 if last_time_close is not None:
                     # fill missing klines
                     interval = 60 * self.p.compression
@@ -103,28 +126,34 @@ class MyLiveFeed(bt.feeds.DataBase):
                     if time_close > last_time_close + interval * 1000:
                         missing_ts = last_time_close + interval * 1000
                         while missing_ts < time_close:
-                            if self.p.debug:
-                                missing_kline_local_time_str = datetime.fromtimestamp(missing_ts / 1000.0, timezone.utc).astimezone().strftime('%Y-%m-%d %H:%M:%S')
-                                stdout_log(f"{self.__class__.__name__}, missing kline time: {missing_kline_local_time_str}")
+                            missing_kline_time = datetime.fromtimestamp(missing_ts / 1000.0, timezone.utc)
+                            if self.is_tradable(missing_kline_time):
+                                if self.p.debug:
+                                    missing_kline_local_time_str = datetime.fromtimestamp(missing_ts / 1000.0, timezone.utc).astimezone().strftime('%Y-%m-%d %H:%M:%S')
+                                    stdout_log(f"{self.__class__.__name__}, missing kline time: {missing_kline_local_time_str}")
 
-                            # 找到缺失的K线 填充策略就是沿用上一根K线的close价格 vol和turnover为0
-                            v = self.hist_data_q.queue[-1]
-                            if v is not None:
-                                new_v = {
-                                    'timeOpen': missing_ts - interval * 1000,
-                                    'timeClose': missing_ts,
-                                    'createTime': 0,    # 约定：只要是沿用的价格数据 createTime和updateTime都为0
-                                    'updateTime': 0,    # 约定：只要是沿用的价格数据 createTime和updateTime都为0
-                                    'symbol': v['symbol'],
-                                    'open': v['close'],
-                                    'high': v['close'],
-                                    'low': v['close'],
-                                    'close': v['close'],
-                                    'vol': 0.0,
-                                    'turnover': 0.0,
-                                    'type': v['type']
-                                }
-                                self.hist_data_q.put(new_v)
+                                # 找到缺失的K线 填充策略就是沿用上一根K线的close价格 vol和turnover为0
+                                v = self.hist_data_q.queue[-1]
+                                if v is not None:
+                                    new_v = {
+                                        'timeOpen': missing_ts - interval * 1000,
+                                        'timeClose': missing_ts,
+                                        'createTime': 0,    # 约定：只要是沿用的价格数据 createTime和updateTime都为0
+                                        'updateTime': 0,    # 约定：只要是沿用的价格数据 createTime和updateTime都为0
+                                        'symbol': v['symbol'],
+                                        'open': v['close'],
+                                        'high': v['close'],
+                                        'low': v['close'],
+                                        'close': v['close'],
+                                        'vol': 0.0,
+                                        'turnover': 0.0,
+                                        'type': v['type']
+                                    }
+                                    self.hist_data_q.put(new_v)
+                            else:
+                                if self.p.debug:
+                                    missing_kline_local_time_str = datetime.fromtimestamp(missing_ts / 1000.0, timezone.utc).astimezone().strftime('%Y-%m-%d %H:%M:%S')
+                                    stdout_log(f"{self.__class__.__name__}, skip non-tradetime missing kline time: {missing_kline_local_time_str}")
                             missing_ts += interval * 1000
 
                 self.hist_data_q.put(result)
@@ -165,14 +194,23 @@ class MyLiveFeed(bt.feeds.DataBase):
                     # 因为_load在同一个K线时间段被调用很多次 所以 需要判断当前时间段所代表的K线是否已经被返回了
                     return  # kline already exists
                 else:
+                    pass
                     # new kline
                     # because market data API denotes Kline by open time, while backtrader denotes Kline by close time
-                    start_time = datetime.fromtimestamp(prev_live_data['timeClose'] / 1000.0, timezone.utc).astimezone()
+                    # start_time = datetime.fromtimestamp(prev_live_data['timeClose'] / 1000.0, timezone.utc).astimezone()
 
         end_time = start_time + timedelta(minutes=self.p.compression)
         if self.p.timeframe == bt.TimeFrame.Seconds:
             end_time = start_time + timedelta(seconds=self.p.compression)
 
+        # 如果当前K线处于不可交易时段 就直接return
+        if not self.is_tradable(end_time.astimezone(pytz.utc)):
+            if end_time.strftime('%Y-%m-%d %H:%M:%S') not in self.dropped_kline_time_list:
+                stdout_log(f"{self.__class__.__name__}, kline time: {end_time.strftime('%Y-%m-%d %H:%M:%S')} is not tradable. Drop")
+                self.dropped_kline_time_list.append(end_time.strftime('%Y-%m-%d %H:%M:%S'))
+            time.sleep(3)
+            return
+
         start_time_str = start_time.strftime('%Y-%m-%d %H:%M:%S')
         end_time_str = end_time.strftime('%Y-%m-%d %H:%M:%S')
 
@@ -218,7 +256,7 @@ class MyLiveFeed(bt.feeds.DataBase):
             else:
                 retry_interval_s = 1
                 if self.p.timeframe == bt.TimeFrame.Seconds:
-                    retry_interval_s = 0.3
+                    retry_interval_s = 0.8
                 time.sleep(retry_interval_s)
 
         # 如果通过API没有获取到K线数据 那就尝试沿用前面的K线来充当
