@@ -374,7 +374,7 @@ def show_perf_graph(backtest_data, is_live=False, backtest_data_dir=None, task_i
 
     # 回测的情形 需要超时杀掉服务
     if not is_live:
-        TIMEOUT_SECONDS = 15
+        TIMEOUT_SECONDS = 120
 
         @app.server.before_request
         def update_last_access_time():
@@ -404,10 +404,38 @@ def show_perf_graph(backtest_data, is_live=False, backtest_data_dir=None, task_i
         update_task_field(task_id, "dash_port", str(port))
         update_task_field(task_id, "last_dash_started_at", datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
 
-        if len(backtest_data["broker_values"]) > 0:
-            return_rate = backtest_data["broker_values"][-1]["value"] / backtest_data["broker_values"][0]["value"] - 1
+        broker_values = backtest_data["broker_values"]
+        orders = backtest_data["orders"]
+        positions = backtest_data["positions"]
+        completed_order_num = 0
+        for symbol in orders.keys():
+            completed_order_num += len([item for item in orders[symbol] if item['order_status'] == "Completed"])
+        update_task_field(task_id, "order_count", str(completed_order_num))
+
+        if len(broker_values) > 0:
+            return_rate = broker_values[-1]["value"] / broker_values[0]["value"] - 1
             update_task_field(task_id, "return_rate", str(return_rate))
 
+        _, _, _, sharpe = get_return_and_sharpe(backtest_data)
+        update_task_field(task_id, "sharpe_ratio", str(sharpe))
+
+        reward_risk_ratio, win_rate, order_avg_return = get_order_pnl_stats(
+            orders=orders,
+            positions=positions,
+            debug=False)
+        update_task_field(task_id, "reward_risk_ratio", str(reward_risk_ratio))
+
+        update_task_field(task_id, "win_rate", str(win_rate))
+        update_task_field(task_id, "order_avg_return", str(order_avg_return))
+
+        commission_cost = get_commission_cost(backtest_data, "9999-99-99 99:99:99")
+        comm_ratio = commission_cost / broker_values[0]['value']
+        update_task_field(task_id, "comm_ratio", str(comm_ratio))
+
+        drawdowns = backtest_data["drawdowns"]
+        max_drawdown = f"{max([item['drawdown'] for item in drawdowns]) / 100.0}"
+        update_task_field(task_id, "max_drawdown", max_drawdown)
+
     if get_self_ip() != "192.168.25.144":
         webbrowser.open(f"http://{get_self_ip()}:{int(port)}")
 
@@ -420,6 +448,33 @@ def show_perf_graph(backtest_data, is_live=False, backtest_data_dir=None, task_i
         use_reloader=False,
         debug=True)
 
+def get_return_and_sharpe(backtest_data):
+    broker_values = backtest_data["broker_values"]
+    bvalue_len = len(broker_values)
+
+    treturns = backtest_data["treturns"]
+    riskfree_rate=backtest_data["riskfree_rate"]
+
+    days_in_year = 252
+
+    bar_interval = 60
+    if backtest_data["timeframe"] == "Seconds":
+        bar_interval = backtest_data["compression"]
+    bars_per_day = (6.5 * 60 * 60) / bar_interval
+
+    total_return = broker_values[-1]['value'] / broker_values[0]['value'] - 1
+    annual_return = "NaN"
+    annual_return = (1 + total_return / (bvalue_len / bars_per_day)) ** days_in_year - 1
+
+    std_per_bar = np.std([item['timereturn'] for item in treturns])
+    std_annual = std_per_bar * np.sqrt(days_in_year * bars_per_day)
+
+    sharpe = "NaN"
+    if std_annual != 0:
+        sharpe = (annual_return - riskfree_rate) / std_annual
+
+    return total_return, annual_return, std_annual, sharpe
+
 # 性能表格的初始化 实际是一个不断触发的定时器
 def init_table_callback(app, debug=False):
     @app.callback(
@@ -440,31 +495,12 @@ def init_table_callback(app, debug=False):
                 use_local_dash_url=backtest_data["use_local_dash_url"],
                 dash_port=dict(backtest_data).get("dash_port", None))
 
-        riskfree_rate = backtest_data["riskfree_rate"]
         broker_values = backtest_data["broker_values"]
-        treturns = backtest_data["treturns"]
         positions = backtest_data["positions"]
         orders = backtest_data["orders"]
 
-        length = len(broker_values)
-        if length > 0:
-            days_in_year = 252
-
-            bar_interval = 60
-            if backtest_data["timeframe"] == "Seconds":
-                bar_interval = backtest_data["compression"]
-            bars_per_day = (6.5 * 60 * 60) / bar_interval
-
-            total_return = broker_values[-1]['value'] / broker_values[0]['value'] - 1
-            annual_return = "NaN"
-            annual_return = (1 + total_return / (length / bars_per_day)) ** days_in_year - 1
-
-            std_per_bar = np.std([item['timereturn'] for item in treturns])
-            std_annual = std_per_bar * np.sqrt(days_in_year * bars_per_day)
-
-            sharpe = "NaN"
-            if std_annual != 0:
-                sharpe = (annual_return - riskfree_rate) / std_annual
+        if len(broker_values) > 0:
+            total_return, annual_return, std_annual, sharpe = get_return_and_sharpe(backtest_data)
 
             completed_order_num = 0
             for symbol in orders.keys():
@@ -479,6 +515,7 @@ def init_table_callback(app, debug=False):
                 completed_limit_order_num += len([item for item in orders[symbol] if item['exec_type'] == "Limit" and item['order_status'] == "Completed"])
 
             commission_cost = get_commission_cost(backtest_data, "9999-99-99 99:99:99")
+            comm_ratio = commission_cost / broker_values[0]['value']
 
             long_positions = []
             short_positions = []
@@ -511,7 +548,7 @@ def init_table_callback(app, debug=False):
                     "平均盈亏比",
                     "交易胜率",
                     "平仓单平均收益率",
-                    "交易手续费",
+                    "交易手续费(损耗占比)",
                     "多头最大持仓量",
                     "空头最大持仓量"
                 ],
@@ -527,7 +564,7 @@ def init_table_callback(app, debug=False):
                     f"{reward_risk_ratio:.8f}" if reward_risk_ratio != float('inf') else 'NaN',
                     f"{win_rate:.8%}" if win_rate is not None else 'NaN',
                     f"{avg_return:.8%}" if avg_return is not None else 'NaN',
-                    f"{commission_cost:.8f}",
+                    f"{commission_cost:.8f}({comm_ratio:.8%})" if commission_cost != 0 else "NaN",
                     f"{max_long_position}",
                     f"{max_short_position}"
                 ]
