@@ -23,7 +23,7 @@ class Logger:
         self.set_log_file()
 
     def set_log_file(self):
-        log_filename = datetime.now().strftime(f"%Y-%m-%d_%H-%M-%S_{self.name}.log")
+        log_filename = datetime.now().strftime(f"%Y-%m-%d_%H-%M-%S.%f_{self.name}.log")
         self.log_filepath = os.path.join(self.log_dir, log_filename)
         file_handler = logging.FileHandler(self.log_filepath)
         formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
